"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
exports.__esModule = true;
var Range = require("../range");
var Position = require("../pos");
var Ast = require("../ast");
var Patch = require("../patch");
var builtInModules = [
    'assert',
    'async_hooks',
    'buffer',
    'child_process',
    'cluster',
    'console',
    'constants',
    'crypto',
    'dgram',
    'dns',
    'domain',
    'events',
    'fs',
    'http',
    'http2',
    'https',
    'inspector',
    'module',
    'net',
    'os',
    'path',
    'perf_hooks',
    'process',
    'punycode',
    'querystring',
    'readline',
    'repl',
    'stream',
    'string_decoder',
    'timers',
    'tls',
    'trace_events',
    'tty',
    'url',
    'util',
    'v8',
    'vm',
    'zlib',
];
function compareImports(a, b) {
    var ap = a.source.value;
    var bp = b.source.value;
    if (typeof ap !== 'string' || typeof bp !== 'string') {
        throw new Error('Import source is not a string');
    }
    return ap.localeCompare(bp);
}
function joinImports(j, nodes) {
    return nodes
        .sort(compareImports)
        .map(function (node) { return j(node).toSource(); })
        .join('\n');
}
var fixer = {
    suggestCodeAction: function (params) {
        var j = params.j, ast = params.ast;
        var importNode = Ast.findFirstNode(ast, j.ImportDeclaration, function (n) { return (n.loc && Range.isInside(params.selection, n.loc)) || false; });
        if (!importNode) {
            return null;
        }
        return {
            title: "Group and sort imports",
            data: null
        };
    },
    createEdit: function (params) {
        var e_1, _a;
        var data = params.data, ast = params.ast, j = params.j;
        var imports = Ast.find(ast, j.ImportDeclaration).nodes();
        var systemImports = [];
        var packageImports = [];
        var sourceImports = [];
        var patches = [];
        try {
            for (var imports_1 = __values(imports), imports_1_1 = imports_1.next(); !imports_1_1.done; imports_1_1 = imports_1.next()) {
                var node = imports_1_1.value;
                if (node.loc) {
                    var loc = {
                        start: node.loc.start,
                        end: { line: node.loc.end.line + 1, column: 0 }
                    };
                    patches.push(Patch.del(loc));
                }
                var path = node.source.value;
                if (typeof path === 'string') {
                    if (builtInModules.includes(path)) {
                        systemImports.push(node);
                    }
                    else if (/^[a-z]/i.test(path)) {
                        packageImports.push(node);
                    }
                    else {
                        sourceImports.push(node);
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (imports_1_1 && !imports_1_1.done && (_a = imports_1["return"])) _a.call(imports_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        patches.reverse();
        var newText = [
            joinImports(j, systemImports),
            joinImports(j, packageImports),
            joinImports(j, sourceImports),
        ]
            .filter(function (a) { return a; })
            .join('\n\n');
        patches.push(Patch.insert(j, Position.create(1, 0), [newText, '\n']));
        return patches;
    }
};
exports["default"] = fixer;
